@@ -176,8 +176,21 @@
  *     - Increase max length of hostname from 64 to 160. (Wolfgang Celeda)
  *   -> 2.33     
  *
- *     TODO: other char devices
+ *   * 2004-03-22: Kurt Garloff <garloff@suse.de>:
+ *     - Allow to switch off sysfs for debugging (-y)
+ *   * 2004-03-31: Kurt Garloff <garloff@suse.de>: 
+ *     - Make sure /dev/scsi/testdev is removed always.
+ *     - Don't overwrite scsi device type when getting serial no. etc. in 
+ *       fill_in_proc()
+ *   -> 2.34.
  *
+ *     TODO: other char devices
+ *     
+ *   * 2004-05-24: Chris Townsend <Chris_Townsend@dra-hq.com>
+ *     - Fix bug with channel specified in scsi.alias
+ *   * 2004-06-09: Kurt Garloff <garloff@suse.de>
+ *     - Handle longer partition names
+ *   -> 2.35.
  */
 
 #include <stdio.h>
@@ -198,7 +211,7 @@
 #include <ctype.h>
 #include <getopt.h>
 
-static char rcsid[] ="$Id: scsidev.c,v 1.28.2.45 2004/02/26 17:00:16 garloff Exp $";
+static char rcsid[] ="$Id: scsidev.c,v 1.28.2.55 2004/09/15 20:36:23 garloff Exp $";
 static char *versid = "scsidev " VERSION " 2004-02-09";
 static char *copyright = "Copyright: GNU GPL  (see file COPYING)\n" \
 " (w)  1994--1997 Eric Youngdale <eric@andante.org>\n"\
@@ -232,6 +245,7 @@ int force = 0;
 int san_del = 0;
 int no_san = 0;
 int no_procscsi = 0;
+int no_sysfs = 0;
 int nm_cbtu = 0;
 int supp_rmvbl = 0;
 int supp_multi = 0;
@@ -1076,15 +1090,18 @@ void findscsidisk(sname * spnt, int no)
 		printf("Findscsidisk: %d\n",no);
 	/* only search up to 16 devices may be a bad assumption, but scanning 
 	 * the whole list could take a long time */
+	unlink (TESTDEV);
 	for (i = no; i < no+16; ++i) {
 		major = disknum_to_sd_major (i);
 		minor = (i << 4) & ~0x0f;
-		unlink (TESTDEV);
      
 		status = mknod (TESTDEV, 0600 | S_IFBLK,
 				makedev (major, minor));
 		fd = open (TESTDEV, O_RDONLY);
+ 		unlink (TESTDEV);
 
+		if (fd < 0)
+			continue;
 		res = ioctl (fd, SCSI_IOCTL_GET_IDLUN, id);
 		if (res < 0)
 			continue;
@@ -1093,8 +1110,6 @@ void findscsidisk(sname * spnt, int no)
 		channel = (id[0] >> 16) & 0xff;
 		lun = (id[0] >> 8 ) & 0xff;
 		scsi_id = id[0] & 0xff;
- 
- 		unlink (TESTDEV);
 		
 		if (verbose >= 2) 
 			printf ("Scanning: %d==%d %d==%d %d==%d %d==%d \n",
@@ -1110,7 +1125,7 @@ void findscsidisk(sname * spnt, int no)
 		}               
 	}
 	/* If we did not find a match, return a good guess.. */ 
-	unlink (TESTDEV);       
+	//unlink (TESTDEV);       
 	spnt->major = disknum_to_sd_major (no);
 	spnt->minor = (no << 4) & ~0x0f;
 }
@@ -1396,7 +1411,7 @@ void create_partitions (sname * spnt)
 	}
 	while (!feof (pf)) {
 		unsigned int maj, min, blk;
-		char nm[8], nm2[8];
+		char nm[64], nm2[64];
 		ptr = fgets (pline, 128, pf);
 		if (!ptr)
 			break;
@@ -1807,9 +1822,12 @@ void fill_in_proc (sname * spnt)
 		spnt->hostname = strdup (spnt->shorthostname);
 	if (!spnt->hostid)
 		spnt->hostid = find_ioport (spnt->shorthostname);
-	
-	spnt->devtp = inq_devtp_to_devtp (spnt->inq_devtp, spnt);
 
+	/* Don't overwrite device type! It should have been filled correctly
+	 * and with matching major/minor before. */
+	// spnt->devtp = inq_devtp_to_devtp (spnt->inq_devtp, spnt);/
+
+	unlink(TESTDEV);
 	fd = mknod (TESTDEV, 0600 | (isblk(spnt->devtp)? S_IFBLK: S_IFCHR),
 		    makedev (spnt->major, spnt->minor));
 	if (fd) {
@@ -1818,11 +1836,14 @@ void fill_in_proc (sname * spnt)
 		return;
 	}
 	fd = open (TESTDEV, O_RDWR | O_NONBLOCK);
+	unlink (TESTDEV);
 	if (fd == -1) {
-	    perror("open");
+	    char buf[64];
+            sprintf(buf, "open %s %03x:%05x",
+		   isblk(spnt->devtp)? "b": "c", spnt->major, spnt->minor); 
+	    perror(buf);
 	    return;
 	}
-	unlink (TESTDEV);
 	inquiry (fd, spnt);
 	get_hsv_os_id(fd, spnt);
 	close (fd);
@@ -1835,6 +1856,7 @@ void fill_in_sg (sname * spnt)
 	int status; int fd;
 
 	errno = 0;
+	unlink (TESTDEV);
 	status = mknod (TESTDEV, 0600 | S_IFCHR,
 			makedev (spnt->major, spnt->minor));
 	if (status) { 
@@ -1914,7 +1936,7 @@ void trigger_module_loads ()
 
 struct sysfsdev {
 	int maj, min;
-	char nm[7];
+	char nm[23];
 	char tp;
 };
 
@@ -1991,7 +2013,7 @@ int sysfs_getinfo (sname * spnt)
 
 void sysfs_parse (sname *spnt, int idx)
 {	
-	char nm[7];
+	char nm[23];
 	char *p1 = sysfsdevs[idx].nm, *p2 = nm;
 	
 	while (!isdigit(*p1) && *p1)
@@ -2037,9 +2059,6 @@ void build_sgdevlist_procscsi ()
 	}
 	fclose (scsifile);
     
-	/* Now, we need to make sure all high-level modules are loaded */
-	trigger_module_loads ();
-
 	scsifile = fopen (PROCSCSI, "r");
 	/* parse /proc/scsi */
 	while (!feof (scsifile)) {
@@ -2231,8 +2250,10 @@ int main (int argc, char * argv[])
 	fprintf(stderr, DEVSCSI " either does not exist, or is not a directory\n");
 	exit(0);
     }
-    while ((c = getopt(argc, argv, "pflLvqshnderoMm:c:A:")) != -1) {
+    while ((c = getopt(argc, argv, "ypflLvqshnderoMm:c:A:")) != -1) {
 	switch (c) {
+	  case 'y':
+	    no_sysfs = 1; break;
 	  case 'p':
 	    no_procscsi = 1; break;
 	  case 'f':
@@ -2275,6 +2296,10 @@ int main (int argc, char * argv[])
 
     if( verbose >= 1 ) 
 	fprintf( stderr, "%s\n", versid );
+    
+    /* Now, we need to make sure all high-level modules are loaded */
+    trigger_module_loads ();
+
     if( force ) 
 	flush_sdev ();
 
@@ -2291,7 +2316,7 @@ int main (int argc, char * argv[])
     register_dev("/dev/scsi/rsth4-334c0i5l0", 9,128, ST, 6, 0x334, 0, 5, 0, -1, "debug", 0, NULL, NULL);
 #else
     if (no_procscsi || try_procscsi ()) {
-	if (find_sysfs ()) {
+	if (no_sysfs || find_sysfs ()) {
 	    if (!quiet) 
 		fprintf (stderr, "/proc/scsi/scsi extensions not found. Fall back to scanning.\n");
 	    build_sgdevlist ();
@@ -2651,7 +2676,7 @@ void build_special ()
 	     */
 	    if( id != -1 && id != spnt->id ) 
 		continue;
-	    if( chan != -1 && id != spnt->chan ) 
+	    if( chan != -1 && chan != spnt->chan )
 		continue;
 	    if( lun != -1 && lun != spnt->lun ) 
 		continue;
